-- displayFunctions
--
-- Author Rival

displayFunctions = {};
function displayFunctions.prerequisitesPresent(specializations)
    return true;
end

function displayFunctions:load(savegame)
    self.playStartUpAnimation = SpecializationUtil.callSpecializationsFunction("playStartUpAnimation");


    self.displayFunctions = {};
    
    local key = "vehicle.displayFunctions.";
    self.displayFunctions.forward = Utils.indexToObject(self.components, getXMLString(self.xmlFile, key.."forward#index"));
    self.displayFunctions.reverse = Utils.indexToObject(self.components, getXMLString(self.xmlFile, key.."reverse#index"));
    self.displayFunctions.blinker1 = Utils.indexToObject(self.components, getXMLString(self.xmlFile, key.."blinker1#index"));
    self.displayFunctions.blinker2 = Utils.indexToObject(self.components, getXMLString(self.xmlFile, key.."blinker2#index"));
    self.displayFunctions.WSH = Utils.indexToObject(self.components, getXMLString(self.xmlFile, key.."wendeschalthebel#index"));
    self.displayFunctions.wshForRot = getXMLString(self.xmlFile, key.."wendeschalthebel#forwardRot");
    self.displayFunctions.wshBackRot = getXMLString(self.xmlFile, key.."wendeschalthebel#reverseRot");
    
    self.hasAttachedTrailers = 0;
    
    -- interior turn lights
    self.interiorTS = {};
    self.interiorTS.lightActive = false;
    self.interiorTS.leftIndex = Utils.indexToObject(self.components, getXMLString(self.xmlFile, "vehicle.turnLights.interiorTurnLights#left"));
    self.interiorTS.rightIndex = Utils.indexToObject(self.components, getXMLString(self.xmlFile, "vehicle.turnLights.interiorTurnLights#right"));
    self.interiorTS.warnIndex = Utils.indexToObject(self.components, getXMLString(self.xmlFile, "vehicle.turnLights.interiorTurnLights#hazard"));
    
    
    self.blinkerHebel = {};
    self.blinkerHebel.index = Utils.indexToObject(self.components, getXMLString(self.xmlFile, key.."blinkerhebel#index"))
    self.blinkerHebel.rotLeft = getXMLString(self.xmlFile, key.."blinkerhebel#rotLeft");
    self.blinkerHebel.rotRight = getXMLString(self.xmlFile, key.."blinkerhebel#rotRight");
    self.blinkerHebel.rotNormal = getXMLString(self.xmlFile, key.."blinkerhebel#rotNormal");
    
    self.startUpAnimation = getXMLString(self.xmlFile, "vehicle.startUpAnimation#name");
    self.firstenginerun = false;
    self.mFlickeringS = 0;
end

function displayFunctions:delete()
end

function displayFunctions:mouseEvent(posX, posY, isDown, isUp, button)
end

function displayFunctions:keyEvent(unicode, sym, modifier, isDown)
end

function displayFunctions:update(dt)
    --local mFlickering=Utils.clamp(math.cos( 7.0 * (g_currentMission.time/1000) ) + 0.2, 0, 1);
    local mFlickering = math.floor((getShaderTimeSec()*7*2 + math.acos(-0.2)) / (math.pi*2));
    if mFlickering ~= self.mFlickeringS then
        self.interiorTS.lightActive = not self.interiorTS.lightActive;
        self.mFlickeringS = mFlickering;
    end
    if self:getIsActive() then
        if self.isMotorStarted then
            if self.interiorTS.leftIndex ~= nil then
                setVisibility(self.interiorTS.leftIndex, (self.interiorTS.lightActive and self.turnLightState == Lights.TURNLIGHT_LEFT));
            end
            if self.interiorTS.rightIndex ~= nil then
                setVisibility(self.interiorTS.rightIndex, (self.interiorTS.lightActive and self.turnLightState == Lights.TURNLIGHT_RIGHT));
            end
            if self.interiorTS.warnIndex ~= nil then
                setVisibility(self.interiorTS.warnIndex, (self.interiorTS.lightActive and self.turnLightState == Lights.TURNLIGHT_HAZARD));
            end
        end
        
            -- Forward / Reverse --
        if self:getLastSpeed() > 1 and self.movingDirection > 0 then
            if self.displayFunctions.forward ~= nil then
                setVisibility(self.displayFunctions.forward, true);
            end
            if self.displayFunctions.WSH ~= nil then
                setRotation(self.displayFunctions.WSH, 0, math.rad(self.displayFunctions.wshForRot), 0);
            end
        elseif self:getLastSpeed() > 1 and self.movingDirection < 0 then
            if self.displayFunctions.reverse ~= nil then
                setVisibility(self.displayFunctions.reverse, true);
            end
            if self.displayFunctions.WSH ~= nil then
                setRotation(self.displayFunctions.WSH, 0, math.rad(self.displayFunctions.wshBackRot), 0);
            end
        else
            if self.displayFunctions.forward ~= nil then
                setVisibility(self.displayFunctions.forward, false);
            end
            if self.displayFunctions.reverse ~= nil then
                setVisibility(self.displayFunctions.reverse, false);
            end
            if self.displayFunctions.WSH ~= nil then
                setRotation(self.displayFunctions.WSH, 0, 0, 0);
            end
        end
        
        if self.isMotorStarted then
            if self.startUpAnimation ~= nil and self.playAnimation ~= nil and not self.firstenginerun then
                self:playStartUpAnimation(1);
				self.firstenginerun = true;
            end
        else
            if self.startUpAnimation ~= nil and self.playAnimation ~= nil and self.firstenginerun then
				self:playStartUpAnimation(-100);
				self.firstenginerun = false;
            end
        end
    end
end

function displayFunctions:updateTick(dt)

    -- blinker light 1 and 2 --
    if table.getn(self.attachedImplements) > 0 or self.hasAttachedTrailers > 0 then
        self.hasAttachedTrailers = 0;
        for i=1, table.getn(self.attachedImplements) do  
            if self.attachedImplements[i].object.turnLights.left[1] ~= nil and self.attachedImplements[i].object.turnLights.right[1] ~= nil then 
                self.hasAttachedTrailers = self.hasAttachedTrailers +1;
            end
            if table.getn(self.attachedImplements[i].object.attachedImplements) > 0 then
                for r=1, table.getn(self.attachedImplements[i].object.attachedImplements) do
                    if self.attachedImplements[i].object.attachedImplements[r].object.turnLights.left[1] ~= nil and self.attachedImplements[i].object.attachedImplements[r].object.turnLights.right[1] ~= nil then 
                        self.hasAttachedTrailers =  self.hasAttachedTrailers +1;
                    end
                end
            end
        end
        if self.displayFunctions.blinker1 ~= nil and self.displayFunctions.blinker2 ~= nil then
            if self.turnLightState ~= 0 then
                if self.hasAttachedTrailers == 0 then
                    setVisibility(self.displayFunctions.blinker1, false);
                    setVisibility(self.displayFunctions.blinker2, false);
                elseif self.hasAttachedTrailers == 1 then
                    setVisibility(self.displayFunctions.blinker1, self.interiorTS.lightActive);
                    setVisibility(self.displayFunctions.blinker2, false);
                elseif self.hasAttachedTrailers == 2 then
                    setVisibility(self.displayFunctions.blinker1, self.interiorTS.lightActive);
                    setVisibility(self.displayFunctions.blinker2, self.interiorTS.lightActive);
                end
            else
                setVisibility(self.displayFunctions.blinker1, false);
                setVisibility(self.displayFunctions.blinker2, false);
            end
        end
    end
    if self.turnLightState ~= nil then
        if self.turnLightState == Lights.TURNLIGHT_LEFT  then
            setRotation(self.blinkerHebel.index, 0, 0, math.rad(self.blinkerHebel.rotLeft));
        elseif self.turnLightState == Lights.TURNLIGHT_RIGHT then
            setRotation(self.blinkerHebel.index, 0, 0, math.rad(self.blinkerHebel.rotRight));
        else
            setRotation(self.blinkerHebel.index, 0, 0, math.rad(self.blinkerHebel.rotNormal));
        end
    end
end

function displayFunctions:draw()
end

function displayFunctions:playStartUpAnimation(speed)
    self:playAnimation(self.startUpAnimation, speed, nil, true);
end